<?php

namespace Freshmail\Core\AdminMenu\Pages;

use Freshmail\Core\Integrations\Integration;

class IntegrationsPage extends AbstractPage
{
  /**
   * Returns page setup data - sections and fields arrays.
   * Can't define as const var because of callbacks.
   *
   * @return array
   * @author MS
   */
  protected function getSetup(): array
  {
    return [
      'subpages' => [
        [
          'parent_slug' => 'freshmail_settings',
          'page_title' => __('Settings', 'freshmail'),
          'menu_title' => __('Settings', 'freshmail'),
          'capability' => 'manage_options',
          'menu_slug' => 'freshmail_integrations',
          'callback' => [$this, 'renderPage'],
          'position' => null,
        ]
      ],
      'sections' => $this->getIntegrationSections(),
      'fields' => $this->getIntegrationFields(),
    ];
  }

  /**
   * Renders this page
   *
   * @return void
   * @author MS
   */
  public function renderPage()
  {
    $data = [];
    include FRERSHMAIL_RESOURCES_PATH . 'views/admin/settings/integrations.php';
  }

  /**
   * Returns fields for all integrations
   *
   * @return array
   * @author MS
   */
  public function getIntegrationFields(): array
  {
    $fields = [];
    foreach (freshmail('integrations') as $integration_name => $integration_obj) {
      if (empty($integration_obj)) {
        continue;
      }

      $integration_fields = $integration_obj->getSettingsFields();
      if (empty($integration_fields)) {
        continue;
      }

      foreach ($integration_fields as &$field) {
        if (empty($field['name'])) {
          continue;
        }
        $defaults = [
          'type' => 'text',
          'id' => $field['name'],
          'css_class' => $integration_name . '-setting-field',
          'value' => $this->{'getValue_' . $field['name']}(empty($field['default_value']) ? '' : $field['default_value']),
          'placeholder' => '',
          'callback' => [$this, 'renderSettingsField_' . $field['name']],
          'page' => 'freshmail-integrations',
          'section' => Integration::OPTION_PREFIX . $integration_name . '_main_settings_section',
          'args' => [],
        ];

        $field = wp_parse_args($field, $defaults);
      }

      $fields = array_merge($fields, $integration_fields);
    }
    return $fields;
  }

  /**
   * Returns sections for all integrations
   *
   * @return array
   * @author MS
   */
  public function getIntegrationSections(): array
  {
    $sections = [];
    foreach (freshmail('integrations') as $integration_name => $integration_obj) {
      if (empty($integration_obj)) {
        continue;
      }

      $integration_sections = $integration_obj->getSettingsSections();

      $defaults = [
        'id' => Integration::OPTION_PREFIX . $integration_name . '_main_settings_section',
        'title' => $integration_name . ' ' . _x('integration', 'default integration setting section name', 'freshmail'),
        'callback' => [$this, 'renderSettingsSection'],
        'page' => 'freshmail-integrations',
      ];

      foreach ($integration_sections as &$section) {
        $section = wp_parse_args($section, $defaults);
      }
      $sections = array_merge($sections, $integration_sections);
    }
    return $sections;
  }
}
