<?php

/**
 * Endpoints: subscriber
 * @see https://freshmail.com/developer-api/subscribers-managing-subscribers/
 *
 * @author MS
 */

namespace Freshmail\Core\Api\Endpoints;

class Subscriber extends AbstractEndpoint
{
  /**
   * Adds subscriber to given list
   *
   * @param string $email
   * @param string $list subscribers list hash
   * @param array $custom_fields
   * @param int $state state of user - acrive, inactive etc. See docs for details.
   * @param int $confirm
   * @return array API response data
   * @author MS
   */
  public function add(string $email, string $list = '', array $custom_fields = [], int $state = 0, int $confirm = -1): array
  {
    if (empty($list)) {
      $list = $this->api_config->getCurrentList();
    }

    if ($state <= 0) {
      $state = $this->api_config->getNewSubscriberStatus();
    }

    if ($confirm < 0) {
      if (2 === $state) {
        $confirm = 1;
      } else {
        $confirm = 0;
      }
    }

    $data = [
      'email' => $email,
      'list' => $list,
      'state' => $state,
      'confirm' => $confirm,
      'custom_fields' => $custom_fields,
    ];
    $json_payload = json_encode($data);
    $result = $this->call('rest/subscriber/add', $json_payload, 'POST');
    return $result;
  }

  /**
   * Edits subscriber at given list
   *
   * @param string $email
   * @param string $list subscribers list hash
   * @param array $custom_fields
   * @param int $state state of user - acrive, inactive etc. See docs for details.
   * @param int $confirm
   * @return array API response data
   * @author MS
   */
  public function edit(string $email, string $list = '', array $custom_fields = [], int $state = 0, int $confirm = 1): array
  {
    if (empty($list)) {
      $list = $this->api_config->getCurrentList();
    }

    $data = [
      'email' => $email,
      'list' => $list,
      'confirm' => $confirm,
      'custom_fields' => $custom_fields,
    ];
    if ($state > 0) {
      $data['state'] = $state;
    }
    $json_payload = json_encode($data);
    $rseult = $this->call('rest/subscriber/edit', $json_payload, 'POST');
    return $rseult;
  }

  public function delete(string $email, string $list = ''): array
  {
    if (empty($list)) {
      $list = $this->api_config->getCurrentList();
    }

    $data = [
      'email' => $email,
      'list' => $list,
    ];
    $json_payload = json_encode($data);
    $result = $this->call('rest/subscriber/delete', $json_payload, 'POST');
    return $result;
  }

  public function get(string $email, string $list = ''): array
  {
    if (empty($list)) {
      $list = $this->api_config->getCurrentList();
    }

    $result = $this->call('rest/subscriber/get/' . $list . '/' . $email);
    return $result;
  }
}
