<?php

namespace Freshmail\Core\Api\Submodules\SubscriberManager;

use Freshmail\CronQueueScheduler\Queue;
use Freshmail\CronQueueScheduler\CronQueueSchedulerType;

class ScheduleHandler implements CronQueueSchedulerType
{
  /**
   * Start timestamp to sync events run - 21.07.2021 01:00:00 GMT+0000
   */
  private const START_TIMESTAMP = 1626829200;

  private ?Queue $queue;

  public function __construct()
  {
    $this->queue = new Queue("fm_subscription_synchronisation_queue", $this);
  }

  public function queueAction(int $item): void
  {
    do_action('fm/cron/unsubscribe_users', $item);
  }

  /**
   * @action fm/cron/queue/schedule_events
   */
  public function scheduleEvents(): void
  {
    if (! wp_next_scheduled('fm/subscription/sync/queue/build')) {
      wp_schedule_event(self::START_TIMESTAMP, 'daily', 'fm/subscription/sync/queue/build');
    }

    if (! wp_next_scheduled('fm/subscription/sync/queue/process')) {
      wp_schedule_event(self::START_TIMESTAMP + 1, 'every_five_minutes', 'fm/subscription/sync/queue/process');
    }
  }

  /**
   * @action fm/subscription/sync/queue/build
   */
  public function buildQueue(): void
  {
    $users = array_map(function (\WP_User $user) {
      return $user->ID;
    }, get_users());

    if (! empty($users)) {
      foreach ($users as $userId) {
        $this->queue->addItem($userId);
      }
    }
  }

  /**
   * @action fm/subscription/sync/queue/process
   */
  public function processQueue(): void
  {
    $this->queue->proceed();
  }

  /**
   * @action fm/plugin/deactivate
   */
  public function clearQueue(): void
  {
    $this->queue->clearItems();
  }

  /**
   * @action fm/plugin/deactivate
   */
  public function descheduleEvents(): void
  {
    wp_clear_scheduled_hook('fm/subscription/sync/queue/build');
    wp_clear_scheduled_hook('fm/subscription/sync/queue/process');
  }
}
