<?php

namespace Freshmail\Core\Api\Submodules\SubscriberManager;

use Freshmail\Core\Api\Models\Subscriber;
use WC_Customer;
use WP_User;
use Freshmail\Core\Api\Submodules\EndpointsManager;

class SubscriptionHandler
{

  protected ?EndpointsManager $endpoints = null;

  private const LISTS_META_NAME = 'f_subscribed_lists';
  private const CONSENT_META_NAME = 'f_api_use_consent';

  public function __construct(EndpointsManager &$endpoints = null)
  {
    $this->endpoints = $endpoints;
  }

  /**
   * @action fm/subscriber/after_add_to_list
   * @action fm/subscriber/after_update_at_list
   */
  public function saveNewSubscribedList(string $list, string $email, array $custom_fields = [], array $additional_args = []): void
  {
    $user = false;
    if (empty($additional_args['user_id'])) {
      $user = get_user_by('email', $email);
    } else {
      $user = get_user_by('ID', $additional_args['user_id']);
    }

    if (false === $user) {
      return;
    }

    $user_lists = get_user_meta($user->ID, self::LISTS_META_NAME, true);
    if (empty($user_lists) || ! is_array($user_lists)) {
      $user_lists = [];
    }

    if (empty($user_lists[$email])) {
      $user_lists[$email] = [];
    }

    $values = array_unique([$list, ...$user_lists[$email]]);
    $user_lists[$email] = $values;
    update_user_meta($user->ID, self::LISTS_META_NAME, $user_lists);
  }

  /**
   * @action updated_user_meta
   */
  public function unsubscribeFromListAfterUncheckingSettingField($meta_id, $object_id, $meta_key, $meta_value): void
  {
    if ('f_api_use_consent' !== $meta_key) {
      return;
    } elseif ('yes' === $meta_value || 1 === intval($meta_value)) {
      return;
    }

      $user = new WC_Customer($object_id);
      $user_lists = get_user_meta($object_id, self::LISTS_META_NAME, true);

      $subscriber = new Subscriber($this->endpoints);
      $subscriber->setEmail($user->get_email());
      $success = $subscriber->unsubscribe();

    if (! $success) {
      error_log('Freshmail INFO: Couldn\'t usubscribe user ID ' . $object_id . ' from list ID ' . $this->endpoints->getApiConfig()->getCurrentList());
      return;
    }

    if (empty($user_lists[$user->get_email()])) {
      return;
    }

      $current_list = $this->endpoints->getApiConfig()->getCurrentList();

    foreach ($user_lists[$user->get_email()] as $key => $list_id) {
      if ($list_id === $current_list) {
          unset($user_lists[$user->get_email()][$key]);
          update_user_meta($object_id, self::LISTS_META_NAME, $user_lists);
          break;
      }
    }
  }

  /**
   * @action fm/cron/unsubscribe_users
   */
  public function handleUnsubscribeByCron(int $user_id): void
  {
      $user = new WC_Customer($user_id);

      $subscriber = new Subscriber($this->endpoints);
      $subscriber->setEmail($user->get_email());
      $subscribed = $subscriber->isEnrolled();

    if ($subscribed || ! in_array(get_user_meta($user_id, self::CONSENT_META_NAME, true), ['yes', '1', 1])) {
      return;
    }

      $user_lists = get_user_meta($user_id, self::LISTS_META_NAME, true);

    if (empty($user_lists[$user->get_email()])) {
      return;
    }

      $current_list = $this->endpoints->getApiConfig()->getCurrentList();

    foreach ($user_lists[$user->get_email()] as $key => $list_id) {
      if ($list_id === $current_list) {
          unset($user_lists[$user->get_email()][$key]);
          update_user_meta($user_id, self::LISTS_META_NAME, $user_lists);
          break;
      }
    }

      update_user_meta($user_id, self::CONSENT_META_NAME, 0);
  }

  /**
   * @action delete_user
   */
  public function unsubscribeFromAllLists(int $user_id, ?int $user_reassign_id, WP_User $user): void
  {
      $user_lists = get_user_meta($user_id, self::LISTS_META_NAME, true);

    if (empty($user_lists)) {
      return;
    }

      $subscriber = new Subscriber($this->endpoints);

    foreach ($user_lists as $email => $lists) {
      if (empty($lists)) {
          continue;
      }

      foreach ($lists as $list_id) {
          $subscriber->setEmail($email);
          $success = $subscriber->unsubscribe($list_id);
        if (! $success) {
          error_log('Freshmail INFO: Couldn\'t usubscribe user ID ' . $user_id . ' from list ID ' . $list_id);
          return;
        }
      }
    }
  }
}
