<?php
/**
 * Core: Config
 *
 * @author PH
 */
namespace Freshmail\Core;

class Config
{
  /**
   * Function initializes plugin config.
   *
   * @action plugins_loaded
   * @author PH
   */
  public function initConfig()
  {
    load_textdomain('freshmail', FRERSHMAIL_RESOURCES_PATH . '/lang/' . get_locale() . '.mo');
  }

  /**
   * Register theme dependencies
   *
   * @action wp_enqueue_scripts 100
   *
   * @link https://developer.wordpress.org/themes/basics/including-css-javascript/
   *
   * @return void
   */
  public function dependencies()
  {
    $version = false !== strpos(get_home_url(), '.test.') ? time() : null;

    // wp_deregister_script('jquery');
    // wp_enqueue_script('jquery', 'https://code.jquery.com/jquery-3.3.1.min.js', array(), '3.3.1', false);

    wp_enqueue_style('FRERSHMAIL/vendor.css', FRERSHMAIL_ASSETS_URI . '/styles/vendor.css', false, $version, 'all');
    wp_enqueue_style('FRERSHMAIL/main.css', FRERSHMAIL_ASSETS_URI . '/styles/main.css', false, $version, 'all');
  }

  /**
   * Register admin backend dependencies.
   *
   * @action admin_enqueue_scripts
   * @author PH
   */
  public function adminDependencies()
  {
    $version = time();

    wp_enqueue_style('FRERSHMAIL/admin.css', FRERSHMAIL_ASSETS_URI . '/styles/admin.css', false, $version);
    wp_enqueue_script('FRERSHMAIL/manifest.js', FRERSHMAIL_ASSETS_URI . '/scripts/manifest.js', ['jquery'], $version, true);
    wp_enqueue_script('FRERSHMAIL/vendor.js', FRERSHMAIL_ASSETS_URI . '/scripts/vendor.js', ['FRERSHMAIL/manifest.js'], $version, true);
    wp_enqueue_script('FRERSHMAIL/admin.js', FRERSHMAIL_ASSETS_URI . '/scripts/admin.js', ['FRERSHMAIL/manifest.js'], $version, true);

    wp_enqueue_script('google-charts', 'https://www.gstatic.com/charts/loader.js');

    wp_localize_script('FRERSHMAIL/admin.js', 'freshmail', $this->localizeScript());
  }

  /**
   * Add settings link on plugins option page
   * @filter plugin_action_links_freshmail/freshmail.php
   */
  public function settingsLink($links)
  {
    $url = esc_url(add_query_arg(
        'page',
        'freshmail_settings',
        get_admin_url() . 'admin.php'
    ));
    $settings_link = "<a href='$url'>" . __('Settings', 'freshmail') . '</a>';
    array_push(
        $links,
        $settings_link
    );
    return $links;
  }

  /**
   * Localize script to for ex. globally declare javascript variables
   *
   * @return array
   */
  private function localizeScript(): array
  {
    global $post;
    $localize = array();

    // $localize['root'] = esc_url_raw(rest_url());
    $localize['ajax'] = admin_url('admin-ajax.php');
    $localize['theme'] = get_bloginfo('template_url');
    $localize['error'] = [
      'emptyField' => __('The field is empty.'),
    ];

    return apply_filters('freshmail_localized_scripts', $localize);
  }
}
