<?php

namespace Freshmail\CronQueueScheduler;

use Freshmail\CronQueueScheduler\CronQueueSchedulerType;

class Queue
{
  private string $key;

  private CronQueueSchedulerType $cronQueueSchedulerObject;

  private int $limit = 50;

  public function __construct(string $key, CronQueueSchedulerType $cronQueueSchedulerObject)
  {
    $this->key = $key;
    $this->cronQueueSchedulerObject = $cronQueueSchedulerObject;
  }

  public function getQueue(): array
  {
    $queue = get_option($this->key);
    if (empty($queue)) {
      $this->initQueue($this->key);
    }
    return ! empty($queue) ? $queue : [];
  }

  private function initQueue(): bool
  {
    return add_option($this->key, []);
  }

  public function addItem(int $item): void
  {
    $queue = $this->getQueue($this->key);
    if (! in_array($item, $queue)) {
      $queue[] = $item;
      update_option($this->key, $queue);
    }
  }

  public function removeItem(int $item): void
  {
    $queue = $this->getQueue($this->key);
    if (in_array($item, $queue)) {
      $queue = array_diff($queue, [$item]);
      update_option($this->key, $queue);
    }
  }

  public function clearItems()
  {
    delete_option($this->key);
  }

  public function proceed(): void
  {
    $index = 0;
    $queue = $this->getQueue();
    if (! empty($queue)) {
      foreach ($queue as $item) {
        if ($index >= $this->limit) {
          break;
        }
        $this->cronQueueSchedulerObject->queueAction($item);
        $this->removeItem($item);
        $index++;
      }
    }
  }

  public function getLimit(): int
  {
    return apply_filters('fm/cron/limit', $this->limit);
  }

  public function setLimit(int $limit): void
  {
    $this->limit = $limit;
  }
}
