<?php

namespace Freshmail\FmDesigner;

use Freshmail\SetupWizard\Section\Validator;
use Freshmail\Core\Api\Api;
use Freshmail\Core\Api\Models\ApiConfig;
use Freshmail\FmDesigner\Integrations\WooCommerce;
use Freshmail\FmDesigner\TemplateRetriever;

class FmDesigner
{
  private ?Api $api = null;
  private ?ApiConfig $apiConfig = null;
  private ?TemplateRetriever $templateRetriever = null;

  public function __construct()
  {
    $this->api = new Api();
    $this->apiConfig = new ApiConfig();
    $this->templateRetriever = createClass('Freshmail\FmDesigner\TemplateRetriever', [$this]);
  }


  /**
   * @action init
   */
  public function init(): void
  {
    /**
     * Can't use freshmail('integrations')->woocommerce->isEnabled() in constructor
     * because it generates infinity loop: "Maximum function nesting level of '256' reached, aborting!".
     */
    if (freshmail('integrations')->woocommerce->isEnabled()) {
      $this->wc = createClass('Freshmail\FmDesigner\Integrations\WooCommerce', [$this]);
    }
  }

  public function getApi(): Api
  {
    return $this->api;
  }

  public function getApiConfig(): ApiConfig
  {
    return $this->apiConfig;
  }

  public function getTemplateRetriever(): TemplateRetriever
  {
    return $this->templateRetriever;
  }

  public static function getEmailTemplateHTML(string $idHash): array
  {
    return freshmail('fmdesigner')->getTemplateRetriever()->getEmailTemplateHTML($idHash);
  }
}
