<?php

namespace Freshmail\FmDesigner\Integrations\WooCommerce;

use Freshmail\FmDesigner\Integrations\WooCommerce;

class EmailTemplatesHandler
{
  private ?WooCommerce $wc = null;

  public function __construct(WooCommerce $wc)
  {
    $this->wc = $wc;
  }

  /**
   * @filter woocommerce_email_classes 10
   */
  public function handle(array $emails): array
  {
    $classes = [
      'WC_Email_New_Order' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailNewOrder',
      'WC_Email_Cancelled_Order' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailCanceledOrder',
      'WC_Email_Failed_Order' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailFailedOrder',
      'WC_Email_Customer_On_Hold_Order' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailCustomerOnHoldOrder',
      'WC_Email_Customer_Processing_Order' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailCustomerProcessingOrder',
      'WC_Email_Customer_Completed_Order' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailCustomerCompletedOrder',
      'WC_Email_Customer_Refunded_Order' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailCustomerRefundedOrder',
      'WC_Email_Customer_Invoice' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailCustomerInvoice',
      'WC_Email_Customer_Note' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailCustomerNote',
      'WC_Email_Customer_Reset_Password' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailCustomerResetPassword',
      'WC_Email_Customer_New_Account' => '\Freshmail\FmDesigner\Integrations\WooCommerce\Emails\EmailCustomerNewAccount',
    ];

    foreach ($classes as $native => $custom) {
      if (! empty($emails[$native]) && ! empty($custom)) {
        $templateID = $emails[$native]->get_option('fm_email_template');
        if (! empty($templateID) && ! in_array($templateID, ['wc_default'])) {
          $emails[$native] = new $custom($templateID, $emails[$native]);
        }
      }
    }

    return $emails;
  }
}
