<?php

namespace Freshmail\FmDesigner\Integrations\WooCommerce;

use WC_Product;

class EmailTemplatesTags
{
  private static function getBasicTags(): array
  {
    $data = [
      '{site_title}' => get_bloginfo('name'),
      '{site_address}' => get_bloginfo('wpurl'),
      '{site_url}' => get_bloginfo('url')
    ];
    return $data;
  }

  public static function getOrderTags(\WC_Order $order): array
  {
    $data = [
      '{item_names}' => self::getOrderItems($order),
      '{customer_username}' => $order->get_user() ? $order->get_user()->data->user_nicename : '',
      '{customer_first_name}' => $order->get_billing_first_name(),
      '{customer_last_name}' => $order->get_billing_last_name(),
      '{customer_name}' => $order->get_billing_first_name() . ' ' . $order->get_billing_last_name(),
      '{customer_email}' => $order->get_billing_email(),
      '{order_number}' => $order->get_order_number(),
      '{order_date}' => wc_format_datetime($order->get_date_created(), (get_option('date_format'))),
      '{order_datetime}' => wc_format_datetime($order->get_date_created(), (get_option('date_format') . ' ' . get_option('time_format'))),
      '{order_subtotal}' => $order->get_subtotal_to_display(),
      '{order_tax}' => $order->get_total_tax(),
      '{order_pay_method}' => $order->get_payment_method(),
      '{order_pay_url}' => $order->get_checkout_payment_url(),
      '{order_billing_address}' => $order->get_formatted_billing_address(),
      '{order_shipping_address}' => $order->get_formatted_shipping_address(),
      '{coupon_code}' => $order->get_coupon_codes() ? implode(', ', $order->get_coupon_codes()) : '',
      '{coupon_code_used}' => $order->get_coupon_codes() ? implode(', ', $order->get_coupon_codes()) : '',
      '{coupon_amount}' => $order->get_discount_to_display(),
      '{dollars_spent_order}' => $order->get_formatted_order_total(),
      '{store_url}' => get_site_url(),
      '{store_url_secure}' => preg_replace("/^http:/i", "https:", get_site_url()),
      '{store_name}' => get_bloginfo('name'),
      '{order_billing_phone}' => $order->get_billing_phone(),
      '{order_shipping_phone}' => $order->get_billing_phone(),
    ];

    return array_merge(self::getBasicTags(), $data);
  }

  public static function getCustomerTags(\WP_User $user): array
  {
    $data = [
      '{customer_username}' => $user->user_login ? : '',
      '{customer_first_name}' => $user->user_firstname ? : '',
      '{customer_last_name}' => $user->user_lastname ? : '',
      '{customer_name}' => $user->display_name ? : '',
      '{customer_email}' => $user->user_email ? : '',
      '{customer_nicename}' => $user->user_nicename ? : '',
      '{customer_display_name}' => $user->display_name ? : '',
      '{user_login}' => $user->user_login ? : '',
      '{user_email}' => $user->user_email ? : ''
    ];
    return array_merge(self::getBasicTags(), $data);
  }

  public static function getNewAccountTags(\WC_Email $email): array
  {
    $data = [
      '{user_password}' => $email->user_pass ? : ''
    ];
    return array_merge(self::getBasicTags(), $data);
  }

  public static function getResetPasswordTags(\WC_Email $email): array
  {
    $data = [
      '{reset_password_link}' => esc_url(add_query_arg(array( 'key' => $email->reset_key, 'id' => $email->user_id ), wc_get_endpoint_url('lost-password', '', wc_get_page_permalink('myaccount'))))
    ];
    return array_merge(self::getBasicTags(), $data);
  }

  public static function getAbandonedCartProductTags($product, int $quantity = 1): array
  {
    if (is_int($product)) {
      $product = new WC_Product($product);
    }
    $data = [
      '{product_name}' => $product->get_title(),
      '{unit_price_tax_incl}' => wc_price(wc_get_price_including_tax($product)),
      '{product_quantity}' => $quantity,
      '{product_cover}' => $product->get_image_id() ? wp_get_attachment_image_url($product->get_image_id(), 'large') : wc_placeholder_img_src('large')
    ];
    return array_merge(self::getBasicTags(), $data);
  }

  private static function getOrderItems(\WC_Order $order): string
  {
    ob_start();

    if (function_exists('wc_get_template')) {
      wc_get_template('emails/email-order-details.php', [
        'order' => $order,
        'sent_to_admin' => false,
        'plain_text' => false,
        'email' => '',
      ]);
    }

    return ob_get_clean();
  }
}
