<?php

namespace Freshmail\FollowUp;

use Freshmail\Core\Integrations\WooCommerce\WooCommerce;

class Settings
{
  /**
   * @filter fm/integrations/woocommerce/settings_fields 10 3
   */
  public function addBirthdayFollowUpSettings(array $fields, string $section, WooCommerce $wc): array
  {
    $fields = array_merge([
      [
        'label' => __('Configuration: Birthday', 'freshmail'),
        'description' => __('Show a field for birthday configuration in profile settings.', 'freshmail'),
        'type' => 'checkbox',
        'name' => $wc->getOptionName('profile_birthday_field'),
        'id' => $wc->getOptionName('profile_birthday_field'),
        'page' => 'freshmail-integrations',
        'section' => $section,
        'args' => ['class' => 'settings-field-row hide-if-section-disabled'],
      ],
      [
        'label' => __('FollowUp: Birthday', 'freshmail'),
        'description' => __('Send birthday follow ups.', 'freshmail'),
        'type' => 'checkbox',
        'name' => $wc->getOptionName('followup_birthday'),
        'id' => $wc->getOptionName('followup_birthday'),
        'page' => 'freshmail-integrations',
        'section' =>  $section,
        'args' => ['class' => 'settings-field-row hide-if-section-disabled'],
      ],
      [
        'label' => __('FollowUp: Abandoned Cart', 'freshmail'),
        'description' => __('Send abandoned cart follow ups.', 'freshmail'),
        'type' => 'checkbox',
        'name' => $wc->getOptionName('followup_abandoned_cart'),
        'id' => $wc->getOptionName('followup_abandoned_cart'),
        'page' => 'freshmail-integrations',
        'section' => $section,
        'args' => ['class' => 'settings-field-row hide-if-section-disabled'],
      ],
      [
        'label' => __('Abandoned Cart Time', 'freshmail'),
        'type' => 'select',
        'default_value' => '24',
        'options' => [
          [
            'label' => __('24 hours', 'freshmail'),
            'value' => 24,
          ],
          [
            'label' => __('48 hours', 'freshmail'),
            'value' => 48,
          ],
          [
            'label' => __('72 hours', 'freshmail'),
            'value' => 72,
          ],
        ],
        'name' => $wc->getOptionName('followup_abandoned_cart_time'),
        'id' => $wc->getOptionName('followup_abandoned_cart_time'),
        'page' => 'freshmail-integrations',
        'section' => $section,
        'args' => ['class' => 'settings-field-row hide-if-section-disabled'],
        'description' => __('Number of hours, since the last activity in cart, after which the user should get the abandoned cart notification.', 'freshmail'),
      ]
    ], $fields);
    return $fields;
  }

  public function isProfileBirthdayFieldEnabled(): bool
  {
    return get_option('f_wc_profile_birthday_field', false);
  }

  public function isFollowUpBirthdayEnabled(): bool
  {
    return get_option('f_wc_followup_birthday', false);
  }

  public function isFollowUpAbandonedCartEnabled(): bool
  {
    return get_option('f_wc_followup_abandoned_cart', false);
  }

  public function getFollowUpAbandonedCartTime(): int
  {
    return get_option('f_wc_followup_abandoned_cart_time', 24);
  }
}
