<?php

namespace Freshmail\FollowUp\Types\AbandonedCart;

use Freshmail\FollowUp\Settings;
use Freshmail\FollowUp\Types\AbandonedCart\CartService;
use Freshmail\FollowUp\Types\AbandonedCart\DataLoader;
use Freshmail\FollowUp\Types\AbandonedCart\AbandonedCartEmail;
use Freshmail\FollowUp\Types\AbandonedCart\UserMeta;
use Freshmail\CronQueueScheduler\CronQueueSchedulerType;
use Freshmail\CronQueueScheduler\Queue;

class AbandonedCart implements CronQueueSchedulerType
{
  private ?Settings $settings = null;

  private ?CartService $cartService = null;
  private ?Queue $queue = null;
  private ?DataLoader $dataLoader = null;

  public function __construct(Settings $settings)
  {
    $this->settings = $settings;

    if ($this->settings->isFollowUpAbandonedCartEnabled()) {
      $this->queue = new Queue("fm_followup_abandonedcart_queue", $this);
      $this->dataLoader = createClass('Freshmail\FollowUp\Types\AbandonedCart\DataLoader', [$this->settings]);
      $this->cartService = createClass('Freshmail\FollowUp\Types\AbandonedCart\CartService');

      add_action('fm/cron/queue/schedule_events', [$this, 'scheduleEvents']);
      add_action('fm/followup/abandonedcart/queue/build', [$this, 'buildQueue']);
      add_action('fm/followup/abandonedcart/queue/process', [$this, 'processQueue']);
      add_action('woocommerce_email_classes', [$this, 'addEmailType'], 9);
    }
  }

  public function queueAction(int $item): void
  {
    if (! empty($item)) {
      do_action('fm/followup/abandonedcart/notification', $item);
      do_action('fm/statistics/abandonedcart/add', $item);

      $userMeta = new UserMeta($item);
      $userMeta->deleteCartLastUpdateTime();
    }
  }

  public function scheduleEvents(): void
  {
    if (! wp_next_scheduled('fm/followup/abandonedcart/queue/build')) {
      wp_schedule_event(time(), 'hourly', 'fm/followup/abandonedcart/queue/build');
    }

    if (! wp_next_scheduled('fm/followup/abandonedcart/queue/process')) {
      wp_schedule_event(time(), 'every_five_minutes', 'fm/followup/abandonedcart/queue/process');
    }
  }

  public function buildQueue(): void
  {
    $users = $this->dataLoader->getUsersWithActiveCart();
    if (! empty($users)) {
      foreach ($users as $userId) {
        $this->queue->addItem($userId);
      }
    }
  }

  public function processQueue(): void
  {
    WC()->mailer();
    $this->queue->proceed();
  }

  public function addEmailType(array $emails): array
  {
    $emails['FM_AbandonedCart_Notification'] = new AbandonedCartEmail($this->dataLoader);
    return $emails;
  }

  /**
   * @action fm/plugin/deactivate
   */
  public function clearQueue(): void
  {
    if (! is_null($this->queue)) {
      $this->queue->clearItems();
    }
  }

  /**
   * @action fm/plugin/deactivate
   */
  public function descheduleEvents(): void
  {
    wp_clear_scheduled_hook('fm/followup/abandonedcart/queue/build');
    wp_clear_scheduled_hook('fm/followup/abandonedcart/queue/process');
  }

  /**
   * @action pre_update_option_f_wc_followup_abandoned_cart
   */
  public function abandonedCartSettingDisabled($value, $oldValue, $option)
  {
    if (! $value) {
      $this->clearQueue();
      $this->descheduleEvents();
    }
    return $value;
  }
}
