<?php

// phpcs:disable PSR1.Methods.CamelCapsMethodName.NotCamelCaps

namespace Freshmail\FollowUp\Types\AbandonedCart;

use Freshmail\FmDesigner\Integrations\WooCommerce\EmailTemplatesTags;
use Freshmail\FmDesigner\FmDesigner;
use Freshmail\FollowUp\Types\AbandonedCart\DataLoader;

class AbandonedCartEmail extends \WC_Email
{
  private ?\WP_User $user = null;
  private ?DataLoader $dataLoader = null;
  private string $templateID;

  public function __construct(DataLoader $dataLoader)
  {
    $this->dataLoader = $dataLoader;

    $this->id = 'abandonedcart';
    $this->title = __('Abandoned Cart', 'freshmail');
    $this->description = __('Abandoned Cart FollowUp message.', 'freshmail');
    $this->customer_email = true;

    add_action('fm/followup/abandonedcart/notification', [$this, 'trigger'], 10, 1);

    parent::__construct();

    $this->templateID = $this->get_option('fm_email_template');
  }

  public function trigger($item): void
  {
    if (! empty($item)) {
      $user = get_user_by('id', $item);
      if (! empty($user)) {
        $this->user = $user;
        $this->setup_locale();

        if ($this->is_enabled()) {
          $this->send($user->user_email, $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments());
        }

        $this->restore_locale();
      }
    }
  }

  public function get_default_subject(): string
  {
    return __('You have abandoned your cart!', 'freshmail');
  }

  public function get_subject(): string
  {
    return ! empty(parent::get_subject()) ? parent::get_subject() : $this->get_default_subject();
  }

  public function get_default_additional_content(): string
  {
    return sprintf(__('Hi %s! We noticed you left something in your cart. Would you like to complete your purchase?', 'freshmail'), '{customer_name}');
  }

  public function get_content_html(): string
  {
    try {
      if (! empty($this->templateID) && ! empty($this->user) && is_a($this->user, 'WP_User')) {
        $this->placeholders = EmailTemplatesTags::getCustomerTags($this->user);

        if ($this->templateID === 'wc_default') {
          return $this->format_string($this->get_additional_content());
        }
        if (empty($content = FmDesigner::getEmailTemplateHTML($this->templateID)) || empty($content['content_reb'])) {
          throw new \Exception("Email template cannot be empty", 1);
        }
        if (empty($productsInCart = $this->dataLoader->getCartItemsByUser($this->user)) || empty($content['product'])) {
          return $this->format_string($content['content_reb']);
        }

        $productsHtmlContent = '';
        foreach ($productsInCart as $productInCart) {
          $this->placeholders = EmailTemplatesTags::getAbandonedCartProductTags($productInCart['id'], $productInCart['quantity']);
          $productsHtmlContent .= $this->format_string($content['product']);
        }

        $this->placeholders = EmailTemplatesTags::getCustomerTags($this->user);
        if (! empty($productsHtmlContent)) {
          $this->placeholders = array_merge(
              EmailTemplatesTags::getCustomerTags($this->user),
              [ '{products_list}' => $productsHtmlContent ]
          );
        }

        return $this->format_string($content['content_reb']);
      }
    } catch (\Throwable $th) {
       error_log($th);
    }
    return parent::get_content_html();
  }

  public function get_content_plain(): string
  {
    return $this->format_string($this->get_additional_content());
  }
}
