<?php

namespace Freshmail\FollowUp\Types\AbandonedCart;

use Freshmail\FollowUp\Types\AbandonedCart\UserMeta;

class CartService
{
  private ?UserMeta $userMeta = null;

  public function __construct()
  {
    if (is_user_logged_in()) {
      $this->userMeta = new UserMeta(get_current_user_id());
    }
  }

  /**
   * @action woocommerce_after_cart_item_quantity_update
   */
  public function afterCartQuantityUpdate(string $cartItemKey, int $quantity, int $oldQuantity, \WC_CART $cart): void
  {
    if ($this->userMeta) {
      if (empty($cart->get_cart())) {
        $this->userMeta->deleteCartLastUpdateTime();
      } else {
        $this->userMeta->setCartLastUpdateTime();
      }
    }
  }

  /**
   * @action woocommerce_add_to_cart
   */
  public function addToCart(): void
  {
    if ($this->userMeta) {
      $this->userMeta->setCartLastUpdateTime();
    }
  }

  /**
   * @action woocommerce_cart_item_removed
   */
  public function cartItemRemoved(string $cartItemKey, \WC_Cart $cart): void
  {
    if ($this->userMeta) {
      if (empty($cart->get_cart())) {
        $this->userMeta->deleteCartLastUpdateTime();
      } else {
        $this->userMeta->setCartLastUpdateTime();
      }
    }
  }
}
