<?php

namespace Freshmail\FollowUp\Types\AbandonedCart;

use Freshmail\FollowUp\Settings;
use WP_User;

class DataLoader
{
  private ?Settings $settings = null;

  public function __construct(Settings $settings)
  {
    $this->settings = $settings;
  }

  public function getUsersWithActiveCart(): array
  {
    return $this->getUsersWithActiveCartResults();
  }

  public function getCartItemsByUser($user): array
  {
    if (is_int($user) && $user > 0) {
      $user = new WP_User($user);
    }
    $userCart = get_user_meta($user->ID, '_woocommerce_persistent_cart_' . get_current_blog_id(), true);
    if (! empty($products = $userCart['cart'])) {
      return array_map(function ($item) {
        return ['id' => $item['product_id'], 'quantity' => $item['quantity']];
      }, $products);
    }
    return [];
  }

  private function getUsersWithActiveCartResults(): array
  {
    $timeInterval = strtotime('-' . $this->settings->getFollowUpAbandonedCartTime() . ' hours');
    $users = new \WP_User_Query([
      'meta_key' => 'fm_cart_last_update_time',
      'meta_value' => $timeInterval,
      'meta_compare' => '<=',
      'fields' => ['ID']
    ]);

    if (! empty($users->get_results())) {
      return array_map(function (object $user) {
        return $user->ID;
      }, $users->get_results());
    }
    return [];
  }
}
