<?php

namespace Freshmail\FollowUp\Types\AbandonedCart;

class UserMeta
{
  private int $userID = 0;
  private const LAST_UPDATE_TIME_FIELD_NAME = 'fm_cart_last_update_time';

  public function __construct(int $userID)
  {
    $this->userID = $userID;
  }

  public function getCartLastUpdateTime(bool $timestamp = false, string $format = ''): string
  {
    $lastUpdateTime = get_user_meta($this->userID, self::LAST_UPDATE_TIME_FIELD_NAME, true) ? : '';
    if (! empty($lastUpdateTime)) {
      if ($timestamp) {
        return $lastUpdateTime;
      } else {
        if (empty($format)) {
          $format = get_option('date_format') . ' ' . get_option('time_format');
        }
        return date($format, $lastUpdateTime);
      }
    }
    return '';
  }

  public function setCartLastUpdateTime(string $time = 'now'): void
  {
    $timestamp = strtotime($time);

    if (get_user_meta($this->userID, self::LAST_UPDATE_TIME_FIELD_NAME, false)) {
      update_user_meta($this->userID, self::LAST_UPDATE_TIME_FIELD_NAME, $timestamp);
    } else {
      add_user_meta($this->userID, self::LAST_UPDATE_TIME_FIELD_NAME, $timestamp);
    }
  }

  public function deleteCartLastUpdateTime(): void
  {
    delete_user_meta($this->userID, self::LAST_UPDATE_TIME_FIELD_NAME);
  }

  /**
   * @action fm/plugin/deactivate
   */
  public function clearAllCartLastUpdateTimes(): void
  {
    delete_metadata('user', 0, self::LAST_UPDATE_TIME_FIELD_NAME, '', true);
  }
}
