<?php

namespace Freshmail\FollowUp\Types\Birthday;

use Freshmail\FollowUp\Types\Birthday\AccountForm;
use Freshmail\FollowUp\Types\Birthday\DataLoader;
use Freshmail\FollowUp\Types\Birthday\BirthdayEmail;
use Freshmail\FollowUp\Settings;
use Freshmail\CronQueueScheduler\CronQueueSchedulerType;
use Freshmail\CronQueueScheduler\Queue;

class Birthday implements CronQueueSchedulerType
{
  private ?AccountForm $accountForm = null;
  private ?DataLoader $dataLoader = null;
  private ?Queue $queue = null;
  private ?Settings $settings = null;

  public function __construct(Settings $settings)
  {
    $this->settings = $settings;

    if ($this->settings->isFollowUpBirthdayEnabled()) {
      $this->queue = new Queue("fm_followup_birthday_queue", $this);
      $this->accountForm = createClass('Freshmail\FollowUp\Types\Birthday\AccountForm', [$this->settings]);
      $this->dataLoader = createClass('Freshmail\FollowUp\Types\Birthday\DataLoader');

      add_action('fm/cron/queue/schedule_events', [$this, 'scheduleEvents']);
      add_action('fm/followup/birthday/queue/build', [$this, 'buildQueue']);
      add_action('fm/followup/birthday/queue/process', [$this, 'processQueue']);
      add_action('woocommerce_email_classes', [$this, 'addEmailType'], 9);
    }
  }

  public function queueAction(int $item): void
  {
    if (! empty($item)) {
      do_action('fm/followup/birthday/notification', $item);
    }
  }

  public function scheduleEvents(): void
  {
    if (! wp_next_scheduled('fm/followup/birthday/queue/build')) {
      wp_schedule_event(time(), 'daily', 'fm/followup/birthday/queue/build');
    }

    if (! wp_next_scheduled('fm/followup/birthday/queue/process')) {
      wp_schedule_event(time(), 'every_five_minutes', 'fm/followup/birthday/queue/process');
    }
  }

  public function buildQueue(): void
  {
    $users = $this->dataLoader->getBirthdayAccounts();
    if (! empty($users)) {
      foreach ($users as $userId) {
        $this->queue->addItem($userId);
      }
    }
  }

  public function processQueue(): void
  {
    WC()->mailer(); // @see https://github.com/woocommerce/woocommerce/issues/19572
    $this->queue->proceed();
  }

  public function addEmailType(array $emails): array
  {
    $emails['FM_Birthday_Notification'] = new BirthdayEmail();
    return $emails;
  }

  /**
   * @action fm/plugin/deactivate
   */
  public function clearQueue(): void
  {
    if (! is_null($this->queue)) {
      $this->queue->clearItems();
    }
  }

  /**
   * @action fm/plugin/deactivate
   */
  public function descheduleEvents(): void
  {
    wp_clear_scheduled_hook('fm/followup/birthday/queue/build');
    wp_clear_scheduled_hook('fm/followup/birthday/queue/process');
  }

  /**
   * @action pre_update_option_f_wc_followup_birthday
   */
  public function birthdaySettingDisabled($value, $oldValue, $option)
  {
    if (! $value) {
      $this->clearQueue();
      $this->descheduleEvents();
    }
    return $value;
  }
}
