<?php

// phpcs:disable PSR1.Methods.CamelCapsMethodName.NotCamelCaps

namespace Freshmail\FollowUp\Types\Birthday;

use Freshmail\FmDesigner\Integrations\WooCommerce\EmailTemplatesTags;
use Freshmail\FmDesigner\FmDesigner;

class BirthdayEmail extends \WC_Email
{
  private ?\WP_User $user = null;
  private string $templateID;
  public function __construct()
  {
    $this->id = 'birthday';
    $this->title = __('Birthday', 'freshmail');
    $this->description = __('Birthday FollowUp message.', 'freshmail');
    $this->customer_email = true;

    add_action('fm/followup/birthday/notification', [$this, 'trigger'], 10, 1);

    parent::__construct();

    $this->templateID = $this->get_option('fm_email_template');
  }

  public function trigger($item): void
  {
    if (empty($item)) {
      return;
    }

    $user = get_user_by('id', $item);

    if (empty($user)) {
      return;
    }

    $this->user = $user;
    $this->setup_locale();

    if ($this->is_enabled()) {
      $this->send($user->user_email, $this->get_subject(), $this->get_content(), $this->get_headers(), $this->get_attachments());
    }

    $this->restore_locale();
  }

  public function get_default_subject(): string
  {
    return __('Happy Birthday!', 'freshmail');
  }

  public function get_subject(): string
  {
    return ! empty(parent::get_subject()) ? parent::get_subject() : $this->get_default_subject();
  }

  public function get_default_additional_content(): string
  {
    return sprintf(__('Best wishes to you %s! We wish you every happiness on your special day!', 'freshmail'), '{customer_name}');
  }

  public function get_content_html(): string
  {
    try {
      if (! empty($this->templateID) && ! empty($this->user) && is_a($this->user, 'WP_User')) {
        $this->placeholders = EmailTemplatesTags::getCustomerTags($this->user);
        if ($this->templateID != 'wc_default') {
          if (! empty($content = FmDesigner::getEmailTemplateHTML($this->templateID))) {
            if (! empty($content['content_reb'])) {
              return $this->format_string($content['content_reb']);
            }
          } else {
            throw new \Exception("Email template cannot be empty", 1);
          }
        } else {
          return $this->format_string($this->get_additional_content());
        }
      }
    } catch (\Throwable $th) {
       error_log($th);
    }
    return parent::get_content_html();
  }

  public function get_content_plain(): string
  {
    return $this->format_string($this->get_additional_content());
  }
}
