<?php

namespace Freshmail\SetupWizard;

use Freshmail\Core\AdminMenu\Pages\ApiSettingsPage;
use Freshmail\Core\AdminMenu\Pages\SetupWizardPage;
use Freshmail\SetupWizard\Section\Input;
use Freshmail\SetupWizard\Section\Validator;

class SetupWizard
{
  private array $pages_raw = [];
  private array $pages = [];

  private ?Validator $validator = null;

  public function __construct()
  {
  }

  /**
   * Init pages
   *
   * @action init
   * @return void
   * @author MS
   */
  public function init()
  {
    if ($this->shouldDisplayWizard()) {
      $this->validator = createClass('Freshmail\\SetupWizard\\Section\\Validator', [freshmail('api')]);
      $this->addWizardPages();

      createClass(SetupWizardPage::class);
    }
  }

  /**
   * @action activated_plugin
   */
  public function cybActivationRedirect($plugin)
  {
    if ($plugin == plugin_basename(FRERSHMAIL_PLUGIN_PATH . 'freshmail.php') && $this->shouldDisplayWizard()) {
      if (! current_user_can('administrator')) {
        exit(wp_redirect(admin_url('admin.php?page=freshmail_unauthorized')));

        die();
      }

        exit(wp_redirect(admin_url('admin.php?page=freshmail_setup_wizard')));
    }
  }

  public function shouldDisplayWizard(): bool
  {
    if (get_option('f_setup_finished', false)) {
      return false;
    }

    return true;
  }

  private function addWizardPages(): void
  {
    $pages = [];

    $pages[1] = [
      'sections' => [
        [
          'slug' => 'conversion',
          // 'modifiers' => ['green'],
          'title' => __('Increase the conversion in your store', 'freshmail'),
          'subtitle' => __('Do you have FreshMail account?', 'freshmail'),
          // 'content' => $this->getInputRadio('test2', ['One' => 1, 'Two' => 2, 'Three' => 3], 2),
          // 'buttons' => [],
        ],
        [
          'slug' => 'bearer',
          'modifiers' => ['green'],
          'title' => __('I have FreshMail account', 'freshmail'),
          'subtitle' => __('Find out where to get the token from', 'freshmail'),
          'content' => $this->getInputPassword('f_bearer_token', 'bearer token', get_option('f_bearer_token', '')),
          'buttons' => [
            [
              'label' => __('Next >', 'freshmail'),
              'data' => ['page' => 'next'],
            ],
          ],
        ],
        [
          'slug' => 'integration',
          'modifiers' => ['white', 'benefits'],
          'title' => __('Find out about the advantages of integrating the module with your account in FreshMail:', 'freshmail'),
          'content' =>
              $this->getMediaText(
                  __('Best deliverability', 'freshmail'),
                  __('Send automatic messages and notifications from the store to your customers via FreshMail\'s secure SMTP. Make sure that your e-mails have the best deliverability and do not fall into spam.', 'freshmail'),
                  ['path' => FRERSHMAIL_ASSETS_PATH . '/images/deliverability.svg', 'alt' => 'Deliverability', 'svg' => true]
              )
              .
              $this->getMediaText(
                  __('Shipping reports', 'freshmail'),
                  __('Track accurate reports on sent messages and monitor their effectiveness. Check the number of openings, clicks on links and bounces to optimize your email marketing. Browse detailed message shipping logs.', 'freshmail'),
                  ['path' => FRERSHMAIL_ASSETS_PATH . '/images/reports.svg', 'alt' => 'Reports', 'svg' => true]
              )
              .
              $this->getMediaText(
                  __('Expert support', 'freshmail'),
                  __('Get the support of the best email marketing specialists7 days a week. We answer every phone call, we do not let customers wait for an e-mail response.', 'freshmail'),
                  ['path' => FRERSHMAIL_ASSETS_PATH . '/images/support.svg', 'alt' => 'Support', 'svg' => true]
              )
              .
              $this->getMediaText(
                  __('Use for free', 'freshmail'),
                  __('Installation of the module is completely free. In addition, in FreshMail you can test the sending of transactional emails for 60 days by sending up to 60,000 messages for free!', 'freshmail'),
                  ['path' => FRERSHMAIL_ASSETS_PATH . '/images/free.svg', 'alt' => 'Free', 'svg' => true]
              ),
        ],
        [
          'slug' => 'create-account',
          'modifiers' => ['gray'],
          'title' => __('I don\'t have account', 'freshmail'),
          'subtitle' => __('Create test account and sent 60 000 mails for free', 'freshmail'),
          // 'content' => $this->getInputCheckbox('test2', 2),
          'buttons' => [
            [
              'label' => __('Create test FreshMail account', 'freshmail'),
              'modifiers' => ['forward'],
              'url' => 'https://freshmail.com/',
              'target' => '_blank',
            ],
          ],
        ],
      ]
    ];

    $pages[] = [
      'sections' => [
        [
          'slug' => 'settings',
          'modifiers' => ['green'],
          'title' => __('Choose your subscribers list', 'freshmail'),
          // 'subtitle' => '',
          'content' => $this->getInputSelect('f_subscribers_list_hash', get_option('f_subscribers_list_hash', ''), ApiSettingsPage::getOptionsForSubscribersList()),
          'buttons' => [
            [
              'label' => __('Next >', 'freshmail'),
              'data' => ['page' => 'next'],
            ],
          ],
        ],
      ]
    ];

    $pages[] = [
      'sections' => [
        [
          'slug' => 'integrations',
          'modifiers' => ['green'],
          'title' => __('Integrations', 'freshmail'),
          // 'subtitle' => '',
        ],
        [
          'slug' => 'smtp',
          'modifiers' => ['gray'],
          'title' => __('SMTP', 'freshmail'),
          'subtitle' => __('Lets you send emails through FreshMail to improve deliverability.', 'freshmail'),
          'content' => $this->getInputCheckbox('f_smtp_enabled', __('Enable SMTP integration', 'freshmail'), get_option('f_smtp_enabled', ''), ! empty(get_option('f_smtp_enabled', '')), 'input --wizard'),
        ],
        [
          'slug' => 'woocommerce',
          'modifiers' => ['white'],
          'title' => __('WooCommerce', 'freshmail'),
          'subtitle' => __('Lets customers enrol to lists during checking out.', 'freshmail'),
          'content' => $this->getInputCheckbox('f_wc_enabled', __('Enable WooCommerce integration', 'freshmail'), get_option('f_wc_enabled', ''), ! empty(get_option('f_wc_enabled', '')), 'input --wizard'),
        ],
        [
          'slug' => 'end',
          'modifiers' => ['green'],
          'buttons' => [
            [
              'label' => __('End configuration >', 'freshmail'),
              'data' => ['page' => 'end'],
            ],
          ],
        ],
      ]
    ];

    $this->pages_raw = $pages;
    $this->pages = $this->sanitizePageData($pages);
  }

  private function getInputCheckbox(string $name, string $label, string $value, bool $checked = false, ?string $class = null)
  {
    $setup = [
      'type' => 'checkbox',
      'value' => $value,
      'class' => $class,
      'options' => [
        'label' => $label,
        'checked' => $checked,
      ],
    ];
    $inputObj = new Input($name, $setup);

    return $inputObj->getHtml();
  }

  private function getInputSelect(string $name, string $value = '', array $options = [], ?string $class = null)
  {
    $setup = [
      'type' => 'select',
      'value' => $value,
      'options' => $options,
      'class' => $class,
    ];
    $inputObj = new Input($name, $setup);

    return $inputObj->getHtml();
  }

  private function getInputPassword(string $name, string $placeholder, string $value = '', ?string $class = null)
  {
    $setup = [
      'type' => 'password',
      'value' => $value,
      'class' => $class,
      'placeholder' => $placeholder,
    ];
    $inputObj = new Input($name, $setup);

    return $inputObj->getHtml();
  }

  private function getMediaText(string $title, string $content, array $image, ?string $class = null)
  {
    $mediaTextObj = new MediaText($title, $content, $image, $class);

    return $mediaTextObj->getHtml();
  }

  private function sanitizePageData(array $pages): array
  {
    foreach ($pages as &$page) {
      foreach ($page['sections'] as &$section) {
        $modifiers = '';
        if (! empty($section['modifiers'])) {
          foreach ($section['modifiers'] as $modifier) {
            $modifiers .= ' --' . $modifier;
          }
        }
        $section['modifiers'] = $modifiers;

        if (! empty($section['buttons'])) {
          foreach ($section['buttons'] as &$button) {
            $modifiers = '';
            if (! empty($button['modifiers'])) {
              foreach ($button['modifiers'] as $modifier) {
                $modifiers .= ' --' . $modifier;
              }
            }
            $button['modifiers'] = $modifiers;

            $data = '';
            if (! empty($button['data'])) {
              foreach ($button['data'] as $prop => $val) {
                $data .= 'data-' . $prop . '="' . $val . '"';
              }
            }
            $button['data'] = $data;
          }
        }
      }
    }
    return $pages;
  }

  /**
   * @action freshmail/setup_wizard/page_selector
   */
  public function renderPageSelector(): void
  {
    $data = [
      'pages' => $this->pages,
    ];
    include FRERSHMAIL_RESOURCES_PATH . 'views/setup-wizard/page-selector.php';
  }

  /**
   * @action freshmail/setup_wizard/pages
   */
  public function renderPages(): void
  {
    $data = [
      'pages' => $this->pages,
    ];
    include FRERSHMAIL_RESOURCES_PATH . 'views/setup-wizard/page.php';
  }

  /**
   * @action freshmail/setup_wizard/page_sections
   */
  public function renderPageSections($page_data, $page_number): void
  {
    foreach ($page_data['sections'] as $section_data) {
      include FRERSHMAIL_RESOURCES_PATH . 'views/setup-wizard/sections/section.php';
    }
  }
}
