<?php

namespace Freshmail\Statistics\Types\AbandonedCart;

use Freshmail\Statistics\Types\AbandonedCart\Data;
use Freshmail\Statistics\Charts\LineChart;

class AbandonedCart
{
  private ?Data $data = null;
  private ?LineChart $lineChart = null;
  
  public function __construct()
  {
    $this->data = new Data();

    if (! $this->data->dbTableExists()) {
      $this->data->createDbTable();
    }

    $this->lineChart = new LineChart(
        __('Number of abandoned carts from last 30 days', 'freshmail'),
        $this->data->getGoogleChartsStatistics(),
        [
          [__('Days', 'freshmail'), 'date'],
          [__('Number of abandoned carts', 'freshmail'), 'number'],
          [__('Amount', 'freshmail'), 'number']
        ]
    );
  }

  /**
   * @action fm/statistics/abandonedcart/add
   */
  public function addStatistic(int $userId): bool
  {
    if (! $userId) {
      return false;
    }

    return $this->data->addAbandonedCartData($userId, $this->data->getCartProductsValueByUser($userId));
  }

  /**
   * @filter fm/admin/pages/statistics/charts/get
   */
  public function renderStatistics($charts): array
  {
    $charts[] = $this->lineChart->render();
    return $charts;
  }
}
