<?php

namespace Freshmail\Statistics\Types\AbandonedCart;

class Data
{
  private $wpdb;
  private const DB_TABLE_NAME = 'fm_abandoned_cart_stats';
  private string $dbTableName = '';

  public function __construct()
  {
    global $wpdb;
    $this->wpdb = $wpdb;
    $this->dbTableName = $this->wpdb->prefix . self::DB_TABLE_NAME;
  }

  public function getGoogleChartsStatistics(int $minusDays = 30): array
  {
    $statistics = [];
    for ($i = $minusDays; $i >= 0; $i--) {
      $date = date("Y-m-d", strtotime("-$i day"));
      $dateJsonFormat = date('Y', strtotime($date)) . ", " . (intval(date('m', strtotime($date)))) . ", " . date('d', strtotime($date));
      if (empty($abandonedCarts = $this->getAbandonedCartsData($date))) {
        $statistics[] = [$dateJsonFormat, 0, 0.00];
        continue;
      }
      $count = 0;
      $amount = 0.00;
      foreach ($abandonedCarts as $abandonedCart) {
        $count++;
        $amount += $abandonedCart['products_value'];
      }
      $statistics[] = [$dateJsonFormat, $count, number_format($amount, 2)];
    }
    return $statistics;
  }

  public function getStatistics(string $format = 'json'): string
  {
    $statistics = $this->getAbandonedCartsData();
    if ($format === 'json') {
      return json_encode($statistics);
    }
    return $statistics;
  }

  public function dbTableExists(): bool
  {
    if ($this->wpdb->get_var("SHOW TABLES LIKE '" . $this->dbTableName  . "'") === $this->dbTableName) {
      return true;
    }
    return false;
  }

  public function createDbTable(): void
  {
    $sql = "CREATE TABLE $this->dbTableName (
                `id` bigint(20) NOT NULL AUTO_INCREMENT,
                `user_id` bigint(20) UNSIGNED NOT NULL DEFAULT '0',
                `date` datetime DEFAULT '0000-00-00 00:00:00' NOT NULL,
                `products_value` decimal(10,2) NOT NULL,
                `currency` varchar(11) NOT NULL,
                UNIQUE KEY `id` (`id`)
            ) " . $this->wpdb->get_charset_collate() . ";";

    require_once(ABSPATH . 'wp-admin/includes/upgrade.php');

    dbDelta($sql);
  }

  public function addAbandonedCartData(int $userId, float $productsValue, string $date = null, string $currency = null): bool
  {
    return $this->wpdb->insert(
        $this->dbTableName,
        [
          'user_id' => $userId,
          'products_value' => $productsValue,
          'date' => $date ?? date('Y-m-d H:i:s', strtotime('now')),
          'currency' => $currency ? : get_woocommerce_currency()
        ],
        [ '%s', '%s', '%s' ]
    );
  }

  public function getAbandonedCartsData($date = false): array
  {
    $query = "SELECT * FROM {$this->dbTableName}";
    $params = [];

    if ($date) {
      $date = date('Y-m-d', strtotime($date));
      $query .= " WHERE DATE(date) = %s";
      $params[] = $date;
    }

    $results = $this->wpdb->get_results($this->wpdb->prepare($query, $params));

    if (! empty($results)) {
      return array_map(function (object $item) {
        return (array) $item;
      }, $results);
    }
    return [];
  }

  public function getCartProductsValueByUser($user): float
  {
    if (is_int($user) && $user > 0) {
      $user = new \WP_User($user);
    }
    $productsValue = 0;
    $userCart = get_user_meta($user->ID, '_woocommerce_persistent_cart_' . get_current_blog_id(), true);
    if (! empty($products = $userCart['cart'])) {
      foreach ($products as $product) {
        if ($productObject = wc_get_product($product['product_id'])) {
          $productsValue += $product['quantity'] * $productObject->get_price();
        }
      }
    }
    return number_format($productsValue, 2);
  }
}
